<?php

namespace Drupal\zipper\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\zipper\Zipper;

class ZipperTablesForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'zipper_tables_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $zid = NULL): array {

    // If this is an edit retrieve the requested record
    $zipdata = NULL;
    if ($zid) {
      $zipper = new Zipper();
      $zipdata = $zipper->get($zid);
    }

    $form['#attached']['library'][] = 'zipper/zipper.styles';
    $form['table_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Table name'),
      '#required' => TRUE,
      '#description' => $this->t('Must be lowercase alphanumeric or underscores and begin with a letter.'),
      '#default_value' => $zipdata['name'] ?? NULL,
    ];

    $form['table_desc'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Description'),
      '#default_value' => $zipdata['description'] ?? NULL,
    ];

    $form['table_type'] = [
      '#type' => 'radios',
      '#title' => 'Relationship',
      '#description' => $this->t('The relationship of the first field to the second.'),
      '#default_value' => $zipdata['relationship'] ?? 1,
      '#options' => [
        TRUE => $this->t('One-to-one'),
        FALSE => $this->t('One-to-many'),
      ],
    ];

    $form['field1'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Field 1 name'),
      '#required' => TRUE,
      '#description' => $this->t('Must be lowercase alphanumeric or underscores and begin with a letter.'),
      '#default_value' => $zipdata['key1'] ?? NULL,
    ];

    $form['field1_type'] = [
      '#type' => 'radios',
      '#title' => $this->t('Field 1 type'),
      '#options' => [
        'number' => $this->t('<div class="field-option"><div class="field-option__thumb"><div class="field-option__icon field-option-number"></div></div><div class="field-option__item">Numeric</div></div>'),
        'text' => $this->t('<div class="field-option"><div class="field-option__thumb"><div class="field-option__icon field-option-text"></div></div><div class="field-option__item">Text</div></div>'),
      ],
      '#default_value' => $zipdata['key_type1'] ?? 'number',
      '#allowed_tags' => ['div'],
    ];

    $form['field2'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Field 2 name'),
      '#required' => TRUE,
      '#description' => $this->t('Must be lowercase alphanumeric or underscores and begin with a letter.'),
      '#default_value' => $zipdata['key2'] ?? NULL,
    ];

    $form['field2_type'] = [
      '#type' => 'radios',
      '#title' => $this->t('Field 2 type'),
      '#options' => [
        'number' => $this->t('<div class="field-option"><div class="field-option__thumb"><div class="field-option__icon field-option-number"></div></div><div class="field-option__item">Numeric</div></div>'),
        'text' => $this->t('<div class="field-option"><div class="field-option__thumb"><div class="field-option__icon field-option-text"></div></div><div class="field-option__item">Text</div></div>'),
      ],
      '#default_value' => $zipdata['key_type2'] ?? 'number',
      '#allowed_tags' => ['div'],
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Create Zipper table'),
    ];

    return $form;
  }

  public function validateForm(array &$form, FormStateInterface $form_state): void {
    parent::validateForm($form, $form_state); // TODO: Change the autogenerated stub

    $table_name = $form_state->getValue('table_name');
    $field1_name = $form_state->getValue('field1');
    $field2_name = $form_state->getValue('field2');

    // Check name format
    if (!$this->isValidName($table_name)) {
      $form_state->setErrorByName('table_name', $this->t('Table name format fails validation.'));
      $form['table_name']['#suffix'] = '<div class="custom-error-message">' . $this->t('The table name must start with a letter and contain only letters, numbers, and underscores.') . '</div>';
   }
    // Check for table name uniqueness
    if ($this->isExistingTable($table_name)) {
      $form_state->setErrorByName('table_name', $this->t('Table name fails validation.'));
      $form['table_name']['#suffix'] = '<div class="custom-error-message">' . $this->t('There is already a database table with this name.') . '</div>';
    }
    // Check field 1 name format
    if (!$this->isValidName($table_name)) {
      $form_state->setErrorByName('field1_name', $this->t('Field name format fails validation.'));
      $form['field1']['#suffix'] = '<div class="custom-error-message">' . $this->t('The field name must start with a letter and contain only letters, numbers, and underscores.') . '</div>';
   }
    // Check field 2 name format
    if (!$this->isValidName($table_name)) {
      $form_state->setErrorByName('field2_name', $this->t('Table name format fails validation.'));
      $form['field2']['#suffix'] = '<div class="custom-error-message">' . $this->t('The field name must start with a letter and contain only letters, numbers, and underscores.') . '</div>';
    }
  }

  public function isValidName(string $name): bool|int {
    return preg_match('/^[a-z][a-z0-9_]*$/', $name);
  }

  public function isExistingTable(string $name): bool {
    return \Drupal::database()->schema()->tableExists($name);
  }

  /**
   * {@inheritdoc}
   * @throws \Exception
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {

    // Create the database table.
    $success = $this->createTable($form_state);
    if ($success) {
      // Create the table entry.
      $zipper = new Zipper();
      $name = $form_state->getValue('table_name');
      $values = $form_state->getValues();
      $zipper->createTable($values);

      \Drupal::messenger()->addMessage('Table ' . $name . ' created.');
    }
    else {
      \Drupal::messenger()->addError('Table creation failed.');
    }
  }

  public function createTable(FormStateInterface $form_state): int|string|\Drupal\Core\Database\StatementInterface|null {

    $name = $form_state->getValue('table_name');
    $field1 = $form_state->getValue('field1');
    $field1_type = $form_state->getValue('field1_type');
    $field2 = $form_state->getValue('field2');
    $field2_type = $form_state->getValue('field2_type');

    $field1_type = $field1_type == 'text' ? 'varchar(255) NOT NULL' : 'int';
    $field2_type = $field2_type == 'text' ? 'varchar(255) NOT NULL' : 'int';

    $db = \Drupal::database();
    return $db->query("CREATE TABLE $name ($field1 $field1_type, $field2 $field2_type)");
  }

}
